@@ -19,6 +19,7 @@ BUS_FILTERED_FILENAME = 'output.filtered.bus'
 BUS_CDNA_PREFIX = 'spliced'
 BUS_INTRON_PREFIX = 'unspliced'
 ECMAP_FILENAME = 'matrix.ec'
+GENE_NAMES_FILENAME = 'genes.names.txt'
 TXNAMES_FILENAME = 'transcripts.txt'
 KB_INFO_FILENAME = 'kb_info.json'
 KALLISTO_INFO_FILENAME = 'run_info.json'
@@ -31,6 +32,8 @@ ADATA_PREFIX = 'adata'
 GENE_NAME = 'gene'
 FEATURE_NAME = 'feature'
 TRANSCRIPT_NAME = 'transcript'
+GENOMEBAM_FILENAME = 'pseudoalignments.bam'
+GENOMEBAM_INDEX_FILENAME = 'pseudoalignments.bam.bai'
 
 UNFILTERED_COUNTS_DIR = 'counts_unfiltered'
 FILTERED_COUNTS_DIR = 'counts_filtered'
@@ -46,6 +49,7 @@ BUS_FILTERED_SUFFIX = '.filtered.bus'
 FLENS_FILENAME = 'flens.txt'
 BATCH_FILENAME = 'batch.txt'
 ABUNDANCE_GENE_FILENAME = 'matrix.abundance.gene.mtx'
+ABUNDANCE_GENE_NAMES_FILENAME = 'matrix.abundance.gene.names.mtx'
 ABUNDANCE_GENE_TPM_FILENAME = 'matrix.abundance.gene.tpm.mtx'
 ABUNDANCE_FILENAME = 'matrix.abundance.mtx'
 ABUNDANCE_TPM_FILENAME = 'matrix.abundance.tpm.mtx'
@@ -71,3 +75,6 @@ CORRECT_CODE = 'c'
 FILTERED_CODE = 'filtered'
 UNFILTERED_CODE = 'unfiltered'
 PROJECT_CODE = 'p'
+
+# Loom
+VELOCYTO_LOOM_NAMES = 'CellID,Gene'
